#include <qlayout.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>

#include "dicproperties.h"

#define DICPATH "/home/zaurus/Documents/application/bedic/"

DicProperties::DicProperties( QWidget *parent, DicPropertiesMode mode ) :
  QDialog( parent, "dic_properties", true ),
  fileNameEdited( false ),
  mode( mode )
{
  setCaption( tr( "Create new dictionary" ) );

  QVBox *vbox;
  QHBox *hbox;
  
  QBoxLayout *l = new QVBoxLayout( this );
  l->setSpacing( 3 );
  l->setMargin( 4 );

  hbox = new QHBox(this);
  l->addWidget( hbox );
  hbox->setSpacing(4);

  vbox = new QVBox(hbox);
  vbox->setSpacing(4);

  QLabel *label = new QLabel(tr("Name"), vbox);
  label = new QLabel(tr("File"), vbox);

  vbox = new QVBox(hbox);
  vbox->setSpacing(4);
  
  nameLE = new QLineEdit( vbox );
  fileLE = new QLineEdit( vbox );

  vbox = new QVBox(this);
  l->addWidget( vbox );
  hbox->setSpacing(2);
  label = new QLabel(tr("Description"), vbox);
  descriptionMLE = new QMultiLineEdit( vbox );

  hbox = new QHBox(this);
  l->addWidget( hbox );
  hbox->setSpacing(4);

  vbox = new QVBox(hbox);
  vbox->setSpacing(4);
  label = new QLabel(tr("Maintainer"), vbox);
  label = new QLabel(tr("Copyright"), vbox);

  vbox = new QVBox(hbox);
  vbox->setSpacing(4);
  
  maintainerLE = new QLineEdit( vbox );
  copyrightLE = new QLineEdit( vbox );

  connect( nameLE, SIGNAL(textChanged(const QString&)), 
    this, SLOT(onNameEdit()) );

  connect( fileLE, SIGNAL(textChanged(const QString&)), 
    this, SLOT(onFileEdit()) );

  if( mode != DP_NEW ) {
    nameLE->setEnabled( false );
    fileLE->setEnabled( false );
  }
  if( mode == DP_SHOW ) {
    descriptionMLE->setEnabled( false );
    maintainerLE->setEnabled( false );
    copyrightLE->setEnabled( false );
  }
  
  onNameEdit();
  
}

void DicProperties::onNameEdit()
{
  if( mode != DP_NEW || fileNameEdited ) return;
  
  QString baseName = nameLE->text().lower().simplifyWhiteSpace();

  //Replace space with underscore
  int pos = 0;
  while( true ) {
    pos = baseName.find( " ", pos );
    if( pos == -1 ) break;
    baseName = baseName.replace( pos, 1, "_" );
  }
  fileLE->setText( QString(DICPATH) + baseName + QString( ".edic" ) );
}

void DicProperties::onFileEdit()
{
  if( !fileLE->hasFocus() ) return; // Only if manually edited
  fileNameEdited = !fileLE->text().isEmpty();
}


