/****************************************************************************
* editor.cpp
*
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* Editor widget for editing dictionary entries
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include <qlayout.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qtopia/resource.h>
#include <qtopia/qpeapplication.h>
#include <qtoolbutton.h>

#include <stdio.h>

#include "editor.h"

ZBEDicEditor::ZBEDicEditor( QWidget *parent ) : QWidget( parent ), isEdited( false )
{

   QBoxLayout *l = new QVBoxLayout( this );
   
   QWidget *toolBar = new QHBox( this );
   l->addWidget( toolBar );
   cancelBT = new QPushButton( tr( "Cancel" ), toolBar );
//   cancelBT->setFlat( true );
   partOfSpeachBT = new QPushButton( tr( "pos" ), toolBar );
   meaningBT = new QPushButton( tr( "mn" ), toolBar );
   exampleBT = new QPushButton( tr( "ex" ), toolBar );
   applyBT = new QPushButton( tr( "Apply" ), toolBar );
//   applyBT->setFlat( true );

   connect(applyBT, SIGNAL(clicked()), 
     this, SLOT(editOK()));
   connect(cancelBT, SIGNAL(clicked()), 
     this, SLOT(editCancel()));

   connect(partOfSpeachBT, SIGNAL(clicked()), 
     this, SLOT(insertPOS()));
   connect(meaningBT, SIGNAL(clicked()), 
     this, SLOT(insertMN()));
   connect(exampleBT, SIGNAL(clicked()), 
     this, SLOT(insertEX()));
   
   int toolbarHeight = font().pixelSize() + 4;
   cancelBT->setMaximumHeight( toolbarHeight );
   partOfSpeachBT->setMaximumHeight( toolbarHeight );
   partOfSpeachBT->setMinimumWidth( 10 );
   meaningBT->setMaximumHeight( toolbarHeight );
   meaningBT->setMinimumWidth( 10 );
   exampleBT->setMaximumHeight( toolbarHeight );
   exampleBT->setMinimumWidth( 10 );
   applyBT->setMaximumHeight( toolbarHeight );

   editorMLE = new DescrEditWidget( this );
   editorMLE->setWordWrap( QMultiLineEdit::WidgetWidth );
   l->addWidget( editorMLE );

   setFocusProxy( editorMLE );
   connect(editorMLE, SIGNAL(editDone()), 
     this, SLOT(editOK()));
}


void ZBEDicEditor::startEditing( const QString &keyword, const QString &description )
{
  this->keyword = keyword;
  editorMLE->setText( description );
  setFocus();
  isEdited = true;
//  ((QPEApplication*)qApp)->grabKeyboard();
}

QString ZBEDicEditor::getKeyword()
{
  return keyword;
}

void ZBEDicEditor::editOK()
{
  if( isEdited ) {
    isEdited = false;
    emit editDone( editorMLE->text() );
  }
}

void ZBEDicEditor::editCancel()
{
  isEdited = false;
  emit editCanceled( );  
}

// void posToLinCol( const QString &text, int pos, int *line, int *col )
// {
//   *line = text.left(pos).contains( "\n" );
//   int np = text.findRev( "\n", pos );
//   if( np == -1 )
//     *col = pos;
//   else
//     *col = pos - np;

//   printf( "lin: %d col: %d\n", *line, *col );
// }

// int linColToPos( const QString &text, int line, int col )
// {
//   int p = 0;
//   while( line > 0 ) {
//     p = text.find( "\n", p );
//     if( p == -1 ) break;
//     p += strlen( "\n" );
//     line--;
//   }
//   p += col;
//   return p;
// }


void findNext( QMultiLineEdit *mle, const char *str )
{
  int line, col;
  mle->getCursorPosition( &line, &col );

  while( true ) {
    QString l = mle->textLine( line );
    if( !l )
      break;

    int np = l.find( str, col );
    if( np != -1 ) {
      col = np + strlen( str );
      printf( "found!" );
      break;
    }
    line++;
    col = 0;
  }
  mle->setCursorPosition( line, col );  
}

void ZBEDicEditor::insertPOS()
{
//   QString text = editorMLE->text();
//   int line, col;
//   editorMLE->getCursorPosition( &line, &col );
//   int p = linColToPos( text, line, col );
  
//   printf( "str: %s\n", text.mid( p, 5 ).latin1() );

//   int np = text.find( "{/s}", p );
//   if( np != -1 ) {
//     printf( "str: %s\n", text.mid( np, 5 ).latin1() );
//     posToLinCol( text, np, &line, &col );
//   } else
//     posToLinCol( text, text.length()-1, &line, &col );
    
//   editorMLE->setCursorPosition( line, col );

  findNext( editorMLE, "{/s}" );
  editorMLE->insert( "\n{s}{ps}{/ps}\n  {ss} {/ss}\n{/s}" );
  editorMLE->cursorWordBackward( false );
  editorMLE->cursorWordBackward( false );
  editorMLE->cursorWordBackward( false );
}

void ZBEDicEditor::insertMN()
{
  findNext( editorMLE, "{/ss}" );
  editorMLE->insert( "\n  {ss} {/ss}" );
  editorMLE->cursorWordBackward( false );
}

void ZBEDicEditor::insertEX()
{
  editorMLE->insert( "{ex} {/ex}" );
  editorMLE->cursorWordBackward( false );
}

void DescrEditWidget::focusOutEvent( QFocusEvent* )
{
//  ((QPEApplication*)qApp)->ungrabKeyboard();
//  printf( "Focus out!\n" );
  emit editDone();
}


void DescrEditWidget::keyPressEvent( QKeyEvent * e )
{
  printf( "key: %d; state: %d\n", e->key(), e->state() );
  if( e->key() == Key_Return && e->state() == ControlButton) 
    emit editDone();
  else
    QMultiLineEdit::keyPressEvent( e );
  
}

