/****************************************************************************
* editor.h
*
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* Editor widget for editing dictionary entries
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#ifndef EDITOR_H
#define EDITOR_H

#include <qwidget.h>
#include <qmultilineedit.h>

class DescrEditWidget;

class ZBEDicEditor: public QWidget {
  
Q_OBJECT

protected:

  DescrEditWidget *editorMLE;
  QString keyword;
  bool isEdited;

  QPushButton *cancelBT;
  QPushButton *partOfSpeachBT;
  QPushButton *meaningBT;
  QPushButton *exampleBT;
  QPushButton *applyBT;

public:

  ZBEDicEditor( QWidget *parent );

  void startEditing( const QString &keyword, const QString &description );
  QString getKeyword();

  /* Solves layout problems */
  virtual QSize sizeHint () const
    {
      return QSize();
    }
  
protected slots:
  void editOK();
  void editCancel();
  void insertPOS();
  void insertMN();
  void insertEX();

signals:
  void editDone(QString);
  void editCanceled();

};


class DescrEditWidget: public QMultiLineEdit
{
  Q_OBJECT
public:
  DescrEditWidget( QWidget *parent ) : QMultiLineEdit( parent )
    {
    }
  
protected:
  virtual void focusOutEvent( QFocusEvent * );
  virtual void keyPressEvent( QKeyEvent * e );

signals:
  void editDone();
  
};


#endif
