/****************************************************************************
* settings.cpp
*
* Copyright (C) 2002 Latchesar Ionkov <lionkov@yahoo.com>
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* This program is based on the kbedic dictionary by 
* Radostin Radnev <radnev@yahoo.com>.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include <stdio.h>

#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qtabdialog.h>
#include <qlistview.h>
#include <qtoolbutton.h>
#include <qvgroupbox.h>
#include <qapplication.h>
#include <qwidgetstack.h>
#include <qtextbrowser.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qdir.h>
#include <qmessagebox.h>
#include <qmultilineedit.h>

#include <qpe/global.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qpe/fontdatabase.h>
#include <qtopia/qcopenvelope_qws.h>
#include <qtopia/qpetoolbar.h>

#include "settings.h"
#include "zbedic.h"
#include "dicproperties.h"

#include <bedic.h>

#include <list>

#include <assert.h>

using namespace std;

class DictItem: public QCheckListItem
{
  StaticDictionary *dict;
  
public:
  DictItem( QListView *parent, bool isActive, StaticDictionary *dict ) :
    QCheckListItem( parent, "", QCheckListItem::CheckBox ),
    dict( dict )
  {
    assert( dict != NULL );

    setText( 0, dict->getName() );
    setText( 1, dict->getFileName() );
    if( !isError() )
      setOn( isActive );      
  }

  void markIntegrityFailed()
  {
    QString fname = dict->getFileName();
    int n = fname.findRev('/');
    QString file = fname.mid(n+1);
    setText( 0, file );
    setEnabled( false );
  }
  
  const char *getFileName() const
  {
    return dict->getFileName();
  }  

  StaticDictionary *getDict() 
  {
    return dict;
  }

  bool isActive() const
  {
    return isOn();
  }

  bool isError() const
  {
    return !isEnabled();
  }
  
  
};


ZBEDicSettings::ZBEDicSettings( QWidget *parent ): QTabDialog( parent, "options", true )
{
  setCaption( tr( "ZBEDic Settings (v. " VERSION ")" ) );

}

void ZBEDic::settings() {
  QHBox* hbox;
  QLabel* label;

  ZBEDicSettings settings(this);
//  settings.setSpacing(0);
//  //  settings.setMargin(0);
  settings.setMinimumWidth( 200 );
  settings.setMinimumHeight( this->height() );

  // ========= Appearance Tab ==========  
  QVBox* appearanceTab = new QVBox(&settings);
  appearanceTab->setSpacing(4);
  appearanceTab->setMargin(4);
//  appearanceTab->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));

  hbox = new QHBox(appearanceTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("Layout"), hbox);
  QComboBox* layoutModeCB = new QComboBox(FALSE, hbox);
  layoutModeCB->insertItem( "Auto" );
  layoutModeCB->insertItem( "Vertical" );
  layoutModeCB->insertItem( "Horizontal" );
  layoutModeCB->setCurrentItem( layoutMode );

  hbox = new QHBox(appearanceTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("List box size"), hbox);
  QComboBox* listSize = new QComboBox(FALSE, hbox);
  listSize->insertItem( "Small" );
  listSize->insertItem( "Medium" );
  listSize->insertItem( "Large" );
  listSize->setCurrentItem( wordListSize );
  
  QCheckBox *toolBarCB[TB_count];
  QGroupBox *gbox = new QVGroupBox( tr( "Tool bar" ), appearanceTab );
  
  for( int i = 0; i < TB_count; i++ ) {
    if( (i % 4) == 0 ) {
      hbox = new QHBox(gbox);
    }
    toolBarCB[i] = new QCheckBox( hbox );
    toolBarCB[i]->setPixmap( action[i]->iconSet().pixmap() );
    toolBarCB[i]->setChecked( actionInToolbar[i] );
  }
  
  hbox = new QHBox(appearanceTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("Font"), hbox);
  settings.flist = new QComboBox(FALSE, hbox);
  QStringList fonts = getFontNames();
  int n = 0;
  for(unsigned int i = 0; i < fonts.count(); i++) {
    settings.flist->insertItem(fonts[i]);
    if (fonts[i] == fontName) {
      n = i;
    }
  }
  settings.flist->setCurrentItem(n);
  settings.fsizes = new QComboBox(FALSE, hbox);
  settings.fsizes->insertItem( QString::number( fontSize ) ); // To set current font
  settings.updateFontSizes();
  connect( settings.flist, SIGNAL(activated(int)), 
    &settings, SLOT(updateFontSizes()) );

  hbox = new QHBox(appearanceTab);
  settings.fontPreviewLB = new QLabel(tr("ABCDEF abcdef 12345"), hbox);
  settings.fontPreviewLB->setFrameStyle( QFrame::Box | QFrame::Sunken );

  connect( settings.flist, SIGNAL(activated(int)), 
    &settings, SLOT(updateFontPreview()) );
  connect( settings.fsizes, SIGNAL(activated(int)), 
    &settings, SLOT(updateFontPreview()) );

  settings.updateFontPreview();
  
  hbox = new QHBox(appearanceTab);
  appearanceTab->setStretchFactor( hbox, 1 );
  
  settings.addTab( appearanceTab, tr("Layout") );

  // ========= Dictionaries Tab ==========
  QVBox* dictTab = new QVBox(&settings); 
  dictTab->setSpacing(2);
  dictTab->setMargin(4);
//  dictTab->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  
  settings.dictList = new QListView( dictTab );
  settings.dictList->addColumn( tr( "Name" ) );
  settings.dictList->addColumn( tr( "File" ) );
//  settings.dictList->setMinimumHeight( fontHeight*8 );

  for( uint i = 0; i < dictionaries.count(); i++ )
    new DictItem( settings.dictList, true, dictionaries.at(i) );
  
//  settings.dictFileLB = new QLabel( QString::null, dictTab );

  hbox = new QHBox(dictTab);  
//  settings.dictActiveLB = new QLabel( QString::null, hbox );
  QPushButton *newdicBT = new QPushButton( QIconSet( Resource::loadPixmap("new") ), tr( "Create new" ), hbox );
  connect( newdicBT, SIGNAL(clicked()), 
    &settings, SLOT(createNewDic()));  
  QPushButton *dictPropertiesBT = new QPushButton( tr( "Properties" ), hbox );
//  dictInfoBT->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  connect( dictPropertiesBT, SIGNAL(clicked()), 
    &settings, SLOT(showDictProperties()));

  settings.searchDictWS = new QWidgetStack( dictTab );
  
  settings.dictSearchBt = new QPushButton( QIconSet( Resource::loadPixmap("find") ), tr( "Search for dictionaries" ), settings.searchDictWS );
  settings.searchDictWS->addWidget( settings.dictSearchBt, 1 );
  settings.dictSearchLB = new QLabel( QString::null, settings.searchDictWS );
  settings.dictSearchLB->setFrameStyle( QFrame::Panel + QFrame::Sunken );
  settings.searchDictWS->addWidget( settings.dictSearchLB, 2 );
  settings.searchDictWS->raiseWidget( settings.dictSearchBt );
//  settings.searchDictWS->setMinimumHeight( settings.dictSearchBt->minimumHeight() );
  // Seems to be bug in QtEmbeeded - QWidgetStack has wrong size policy
  settings.searchDictWS->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  
//  dictSearchBt->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  connect( settings.dictSearchBt, SIGNAL(clicked()), 
    &settings, SLOT(searchForDictionaries()));
//   connect( settings.dictList, SIGNAL(currentChanged(QListViewItem*)), 
//     &settings, SLOT(dictSelected(QListViewItem*)));
//   connect( settings.dictList, SIGNAL(clicked(QListViewItem*)), 
//     &settings, SLOT(dictSelected(QListViewItem*)));
  connect( settings.dictList, SIGNAL(returnPressed(QListViewItem*)), 
    &settings, SLOT(toggleActive(QListViewItem*)));
  
//   if( settings.dictList->firstChild() != NULL ) {
//     settings.dictList->setCurrentItem(settings.dictList->firstChild());
//   }  
  
  settings.addTab( dictTab, tr("Dictionaries") );
  
  // ========= Advanced Tab ==========
  QVBox* advancedTab = new QVBox(&settings);
  advancedTab->setSpacing(4);
  advancedTab->setMargin(4);
//  advancedTab->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  
  hbox = new QHBox(advancedTab);
  hbox->setSpacing(4);
  
  label = new QLabel(tr("Words in list"), hbox);
  QSpinBox* llen = new QSpinBox(8, 500, 1, hbox);
  llen->setValue(listLen);
        
  hbox = new QHBox(advancedTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("History length"), hbox);
  QSpinBox* hlen = new QSpinBox(8, 100, 1, hbox);
  hlen->setValue(historyLen);

  QPushButton *clearHistBt = new QPushButton( tr( "Clear history" ), advancedTab );
//  dictSearchBt->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  connect( clearHistBt, SIGNAL(clicked()), 
    this, SLOT(clearHistory()));
  
//   hbox = new QHBox(advancedTab);
//   hbox->setSpacing(4);
//   hbox->setMargin(2);
//   label = new QLabel(tr("Dictionary Path"), hbox);
//   QString dp;
//   for(unsigned int i = 0; i < dictPaths.count(); i++) {
//     dp += dictPaths[i] + ":";
//   }
//   QLineEdit* dpath = new QLineEdit(dp, hbox);
  
  QCheckBox* spron = new QCheckBox(tr("Show pronounciation"), advancedTab);
  spron->setChecked(showPron);

  hbox = new QHBox(advancedTab);
  advancedTab->setStretchFactor( hbox, 2 );
  
  settings.addTab( advancedTab, tr("Misc") );
  
//  appearanceTab->adjustSize();
  settings.adjustSize();
  settings.setOkButton( QString() );
  
  settings.searchInLocation = searchInLocation;
  settings.searchInAllDocs = searchInAllDocs;
  
  if (settings.exec() == QDialog::Accepted) {
    listLen = llen->value();
    historyLen = hlen->value();
//     dictPaths=QStringList::split(':', dpath->text());

    // Update toolbar
    for( int i = 0; i < TB_count; i++ ) {
      if( actionInToolbar[i] )
        action[i]->removeFrom( mainToolBar );
    }
    for( int i = 0; i < TB_count; i++ ) {
      actionInToolbar[i] = toolBarCB[i]->isChecked();
      if( actionInToolbar[i] ) {
        action[i]->addTo( mainToolBar );
      }
    }    
    
    QString fn = settings.flist->currentText();
    int fs = settings.getFontSize();
    if (fn!=fontName || fs!=fontSize) {
      fontName = fn;
      fontSize = fs;
      changeFont(fn, fs);
    }

    showPron = spron->isChecked();
    descrFormatter->setShowPron(showPron);
                
    layoutMode = layoutModeCB->currentItem();
    wordListSize = listSize->currentItem();
    updateLayout( wordListSize, layoutMode );
    mainLayout->activate();

    bool currentInactive = false;
    dictionaries.clear();
    for( QListViewItem *item = settings.dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
      DictItem *dictItem = (DictItem*)item;
      if( dictItem->isActive() ) {
        dictionaries.inSort(dictItem->getDict());
      } else {
        if( dictItem->getDict() == dict ) {
          currentInactive = true;
          dict = NULL;
        }
        delete dictItem->getDict();
      }
    }

    fillInDictPopup();

    if( currentInactive || dict == NULL ) { // If currect dict is inactive or no dict slected
      StaticDictionary *d = NULL;
      if (dictionaries.count() > 0) {
        d = dictionaries.at(0);
      }
      setDictionary( d );
    }
    
  } else {
    // Delete loaded, but inactiv dictionaries
    for( QListViewItem *item = settings.dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
      DictItem *dictItem = (DictItem*)item;
      if( dictionaries.find( dictItem->getDict() ) == -1 )
        delete dictItem->getDict();
    }
  }

  searchInLocation = settings.searchInLocation;
  searchInAllDocs = settings.searchInAllDocs;
}

//=========== Font manipulation =============

void ZBEDicSettings::updateFontSizes()
{
  FontDatabase fd;
  QValueList<int> pointSizes = fd.pointSizes( flist->currentText() );
  QValueList<int>::Iterator it;
  int fontBugFix = pointSizes[0]>=70 ? 10 : 1;
  int previousSize = getFontSize();
  fsizes->clear();
  int bestFit = 300;
  int bestItem = 0, currentItem = 0;
  for( it = pointSizes.begin(); it != pointSizes.end(); it++, currentItem++ ) {
    int itSize = *it / fontBugFix;
    fsizes->insertItem( QString::number( itSize ) );
    if( abs( previousSize - itSize ) < bestFit ) {
      bestFit = abs( previousSize - itSize );
      bestItem = currentItem;
    }
  }
  fsizes->setCurrentItem( bestItem );
}

int ZBEDicSettings::getFontSize()
{
  return fsizes->currentText().toInt();
}

void ZBEDicSettings::updateFontPreview()
{
  QString fn = flist->currentText();
  int fs = getFontSize();

  fontPreviewLB->setFont( QFont( fn, fs ) );
}

void ZBEDic::zoomIn()
{
  FontDatabase fd;
  QValueList<int> pointSizes = fd.pointSizes( fontName );
  int fontBugFix = pointSizes[0]>=70 ? 10 : 1;
  QValueList<int>::Iterator it = pointSizes.find( fontSize*fontBugFix );
  if( it == pointSizes.end() ) return;
  it++;
  if( it == pointSizes.end() ) return;
  changeFont( fontName, *it / fontBugFix );
}

void ZBEDic::zoomOut()
{
  FontDatabase fd;
  QValueList<int> pointSizes = fd.pointSizes( fontName );
  int fontBugFix = pointSizes[0]>=70 ? 10 : 1;
  QValueList<int>::Iterator it = pointSizes.find( fontSize*fontBugFix );
  if( it == pointSizes.end() || it == pointSizes.begin() ) return;
  it--;
  changeFont( fontName, *it / fontBugFix );
}

  
// ================ Search for dictionaries ================

SearchDictFilesDialog::SearchDictFilesDialog( QWidget *parent,
  QString searchInLocation, bool searchInAllDocs  ) :
  QDialog( parent, "search_location", true )
{
  setCaption( tr( "Search location" ) );

//  setMinimumWidth( 200 );
  // -4 since the dialog has a frame
//  setMinimumHeight( parentWidget()->height()-4 );
  
  QVBox* vbox = new QVBox( this );
  
  vbox->setSpacing(4);
  vbox->setMargin(4);

  searchIn = new QButtonGroup( 1, Horizontal, tr( "Search dictionaries in" ), vbox );

//  QGroupBox *groupBox = new QGroupBox( 1, Horizontal, tr( "Search in" ), vbox );

//  QVBox* grvbox = new QVBox( groupBox );
  
  new QRadioButton( tr( "All documents" ), searchIn );
  new QRadioButton( tr( "Selected location" ), searchIn );
  searchInLocationLE = new QLineEdit( searchInLocation, searchIn );
  searchInLocationLE->setEnabled( searchInAllDocs ? false : true );

  connect( searchIn, SIGNAL(clicked(int)), 
    this, SLOT(searchInClicked(int)) );
  
  searchIn->setButton( searchInAllDocs ? 0 : 1 );
  
  QPushButton *searchBt = new QPushButton( tr( "Search for dictionaries" ), vbox );
  connect( searchBt, SIGNAL(clicked()), 
    this, SLOT(startSearch()) );
  
  QBoxLayout * l = new QVBoxLayout( this );
  l->addWidget( vbox );

  adjustSize();
}

void SearchDictFilesDialog::searchInClicked( int button )
{
  searchInLocationLE->setEnabled( button == 1 ? true : false );
}

void SearchDictFilesDialog::startSearch( )
{
  accept();
}

static void searchFilesInDir( list<QString> &dictFileNames, const QString &searchInLocation,
  const char* pattern )
{
    QDir dzdir( searchInLocation, pattern, QDir::Name,
      QDir::Files | QDir::Readable);
  
    const QFileInfoList* filist = dzdir.entryInfoList();
    if (filist != 0) {
      QFileInfoListIterator fit(*filist);
      QFileInfo* fi;
    
      while ((fi=fit.current())) {
        QString fname = fi->absFilePath();
//         struct stat st;
//         if(stat(fname.local8Bit(), &st) != 0) {
//           continue;
//         }
      
        dictFileNames.push_back( fname );      
        ++fit;
      }
    }
}

void ZBEDicSettings::searchForDictionaries()
{
//  QCopEnvelope busyStart( "QPE/System", "busy()" );
//  qApp->processEvents();


  SearchDictFilesDialog dialog( this, searchInLocation, searchInAllDocs );

  if( dialog.exec() == QDialog::Rejected )
    return;

  searchInAllDocs = dialog.searchIn->id( dialog.searchIn->selected() ) == 0 ? true : false;
  searchInLocation = dialog.searchInLocationLE->text();
  
  searchDictWS->raiseWidget( dictSearchLB );
  dictSearchLB->setText( tr( "Searching for dictionaries..." ) );
  qApp->processEvents();

  list<QString> dictFileNames;
  
  if( searchInAllDocs ) {
    
    DocLnkSet fset;
    Global::findDocuments(&fset, "application/bedic");
    QList<DocLnk> flist = fset.children();
    for(unsigned int ii = 0; ii < flist.count(); ii++) {
      const DocLnk* f = flist.at(ii);
      dictFileNames.push_back( f->file() );
    }
    
  
  } else {
    searchFilesInDir( dictFileNames, searchInLocation, "*.hdic" );    
    searchFilesInDir( dictFileNames, searchInLocation, "*.dic.dz" );
    searchFilesInDir( dictFileNames, searchInLocation, "*.edic" );    
  }

  // Remove dic.dz associated with .hdic
  for( list<QString>::iterator it = dictFileNames.begin();
       it != dictFileNames.end(); it++ ) {
    const QString fname = *it;
    if( fname.right(5) != ".hdic" )
      continue;

    const QString base_name = fname.left(fname.length()-5);
    
    for( list<QString>::iterator it2 = dictFileNames.begin();
         it2 != dictFileNames.end(); it2++ ) {
      const QString fname2 = *it2;
      if( fname2.left(fname2.length()-7) == base_name ) {
        dictFileNames.erase( it2 );
        break;
      }
      
    }
  }
  
  
  for( list<QString>::iterator it = dictFileNames.begin();
       it != dictFileNames.end(); it++ ) {
    const QString fname = *it;

    bool itemExist = false;
    for( QListViewItem *item = dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
      DictItem *dictItem = (DictItem*)item;
      if( dictItem->getFileName() == fname ) {
        itemExist = true;
        break;
      }
    }
    
    if( !itemExist ) {
      string errorMessage;
      StaticDictionary* d = StaticDictionary::loadDictionary(fname, false, errorMessage);
      if( d == NULL )
        ; // TODO
      else {
        DictItem *item = new DictItem( dictList, false, d );
        dictList->insertItem( item );
      }
    } 
  }

  dictSearchLB->setText( tr( "Checking integrity..." ) );
  qApp->processEvents();
  
  for( QListViewItem *item = dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
    DictItem *dictItem = (DictItem*)item;
    if( !dictItem->getDict()->checkIntegrity() )
      dictItem->markIntegrityFailed();
  }

  searchDictWS->raiseWidget( dictSearchBt );
//  qApp->processEvents();
  
//  QCopEnvelope busyEnd( "QPE/System", "notBusy()" );
}


// void ZBEDicSettings::dictSelected( QListViewItem *item )
// {
//   if( item == NULL ) return;
  
//   DictItem *ditem = (DictItem*)item;
//   dictFileLB->setText( ditem->getFileName() );

//   QString statusText = tr( "Status: " );
//   if( ditem->isError() ) {
//     statusText += ditem->getDict()->getErrorMessage();
//   } else {
//     statusText += ditem->isActive() ? tr( "Active" ) : tr( "Inactive" );
//   }
//   dictActiveLB->setText( statusText );
// }

void ZBEDicSettings::toggleActive( QListViewItem *item )
{
  QCheckListItem *citem = (QCheckListItem*)item;
  if( citem->isEnabled() )
    citem->setOn( !citem->isOn() );
//  dictSelected( item );
}

// class needed to handle resize events
class DictInfoDialog: public QDialog
{
  QTextBrowser *info;
  QVBox* main;
public:
  DictInfoDialog( QWidget *parent ) :
    QDialog( parent, "Dict Info dialog", true )
  {
    main = new QVBox(this);
    main->setSpacing(4);
    main->setMargin(2);
//    main->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, false));
    info = new QTextBrowser( main );
//    info->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, false));
//    setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, false));
//    setSizePolicy(QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum, false));
  }

  void setInfoText( QString &text )
  {
    info->setText( text );
    updateGeometry();
    main->setMinimumSize( size() );
    main->setMaximumSize( size() );    
    main->adjustSize();
  }
  

protected:
//   void resizeEvent ( QResizeEvent *e )
//   {
//     QDialog::resizeEvent( e );
//     main->setMaximumSize( e->size() );
//     main->adjustSize();
//   }
  
};

void ZBEDicSettings::createNewDic()
{
  DicProperties dic_properties( this, DP_NEW );

  while( true ) {
    if( dic_properties.exec() == QDialog::Rejected )
      return;

    const QString name = dic_properties.nameLE->text().stripWhiteSpace(),
      file = dic_properties.fileLE->text().stripWhiteSpace();

    if( name.length() == 0 ) {
      QMessageBox::warning(this, tr( "Creating dictionary"), tr("You must specify a unique name") );
      continue;
    }

    QString fileName = dic_properties.fileLE->text().utf8();
    // Check if directories exist
    int index = 0, new_index;
    bool cancel = false;
    while( true ) {
      new_index = fileName.find( "/", index );
      if( new_index == -1 ) break;
      if( new_index - index > 0 ) {
        QDir dir( fileName.left( new_index ) );
        if( !dir.exists() ) {
          int ret = QMessageBox::warning(this, tr( "Creating dictionary" ), tr("Directory: ") + dir.path() +
            tr( "\ndoes not exist. Should I create it?" ), tr( "Yes" ), tr( "Cancel" ) );
          if( ret == 1 ) {
            cancel = true;
            break;
          } else {
            // Create dir
            bool success = QDir::current().mkdir( dir.path() );
            if( !success ) {
              QMessageBox::warning(this, tr( "Creating dictionary" ), tr("Creating directory has failed!") );
              cancel = true;
              break;
            }
          }
        }
      }
      index = new_index+1;
    }
    if( cancel ) continue;
    
    string errorMessage;
    DynamicDictionary *new_dic = createSQLiteDictionary( fileName,
      dic_properties.nameLE->text().latin1(), errorMessage );

    if( new_dic == NULL ) {
      QMessageBox::warning(this, tr( "Creating dictionary failed" ), tr("Dictionary error:\n") + errorMessage.c_str() );
    } else {
      setMetaData( &dic_properties, new_dic );

      // Add to the list of dictionaries
      DictItem *item = new DictItem( dictList, false, new_dic );
      dictList->insertItem( item );
      dictList->setSelected( item, true );
      item->setOn( true );
      dictList->ensureItemVisible( item );
      
      QMessageBox::information(this, tr( "Success" ), QString(tr("Dictionary '")) + name + QString(tr("' successfully created")) );
      break;
    }
  }
  
}

void ZBEDicSettings::setMetaData( DicProperties *dic_properties, DynamicDictionary *dic )
{
  bool success = true;
  success &= dic->setProperty( "description", dic_properties->descriptionMLE->text().utf8() );
  success &= dic->setProperty( "maintainer", dic_properties->maintainerLE->text().utf8() );
  success &= dic->setProperty( "copyright", dic_properties->copyrightLE->text().utf8() );
  if( !success ) {
    QMessageBox::warning(this, tr( "zbedic: DB Error" ),
      tr("Updating dictionary properties  has failed:\n") + dic->getErrorMessage() );
  }
}


void ZBEDicSettings::showDictProperties()
{
  DictItem *ditem = (DictItem*)dictList->selectedItem();
  if( ditem == NULL ) return;

  StaticDictionary *dic=ditem->getDict();
  if( dic == NULL )
    return;
  
  DicProperties dic_properties( this, dic->isMetaEditable() ? DP_EDIT : DP_SHOW );
  
  dic_properties.nameLE->setText( dic->getName() );
  dic_properties.fileLE->setText( dic->getFileName() );

  string str;
  if( dic->getProperty( "description", str ) ) {
    dic_properties.descriptionMLE->setText( str.c_str() );
  }
  if( dic->getProperty( "maintainer", str ) ) {
    dic_properties.maintainerLE->setText( str.c_str() );
  }
  if( dic->getProperty( "copyright", str ) ) {
    dic_properties.copyrightLE->setText( str.c_str() );
  }
  
  if( dic_properties.exec() == QDialog::Rejected )
    return;

  if( dic->isDynamic() && dic->isMetaEditable() ) {
    setMetaData( &dic_properties, (DynamicDictionary*)dic  );
  }
}


void ZBEDic::clearHistory()
{
  wordBox->listBox()->clear();
  wordBox->listBox()->setCurrentItem( -1 );
  updateNavigationButtons();
}

QStringList ZBEDic::getFontNames() {
	FontDatabase fd;
	return fd.families();
}
