/****************************************************************************
* zbedic.cpp
*
* Copyright (C) 2002 Latchesar Ionkov <lionkov@yahoo.com>
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* This program is based on the kbedic dictionary by 
* Radostin Radnev <radnev@yahoo.com>.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include <qwidget.h>
#include <qmainwindow.h>
#include <qapplication.h>
#include <qfont.h>
#include <qclipboard.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qtextview.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qstring.h>
#include <qmenubar.h>

#include <qmenubar.h>
#include <qlayout.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qsizepolicy.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qvgroupbox.h>
#include <qtextbrowser.h>
#include <qlist.h>
#include <qdir.h>
#include <qaccel.h>

#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>
#include <qtopia/resource.h>
#include <qpe/global.h>
#include <qpe/applnk.h>
#include <qtopia/qpemenubar.h>
#include <qtopia/qpetoolbar.h>
#include <qtopia/qpeapplication.h>


#include <sys/time.h>

#include "zbedic.h"
#include "utf8.h"
#include "keyboard_shortcuts.h"
#include "editor.h"

#include <assert.h>

using namespace std;

//void zprint(void);

ZBEDic::ZBEDic(QWidget *parent, const char *name, WFlags f) :
	QMainWindow(parent, name, f), dict( NULL ), onWordLookup( false ),
        fullyInitialized(false) {

  Config conf("zbedic");
  conf.setGroup("general");

  // Read data from ini file and set it
  layoutMode = conf.readNumEntry("layout-mode", 0);
  listLen = conf.readNumEntry("list-length", 25);
  wordListSize = conf.readNumEntry("word-list-size", 1);
  historyLen = conf.readNumEntry("history-length", 30);
  showPron = conf.readBoolEntry("show-pron", true);
  previousDict = conf.readEntry("previous-dictionary", QString::null);

  searchInLocation = conf.readEntry("search-dict-loc", "/var/mnt/card/Documents/application/bedic" );
  searchInAllDocs = conf.readBoolEntry("search-dict-all", true);

//  fprintf( stderr, "DeviceID: %s\n", (const char*)Global::deviceId() );

  // Set font, read from file settings and set it
  fontName = conf.readEntry("font-name", "unifont");
  fontSize = conf.readNumEntry("font-size", 11);
  font = QFont(fontName, fontSize);
  updateFont(font);

// 	dictPaths = conf.readListEntry("dict-path", ':');
// 	if (dictPaths.count() == 0) {
// 		dictPaths.append("/opt/QtPalmtop/share/zbedic");
// 		dictPaths.append("/mnt/cf/opt/QtPalmtop/share/zbedic");
// 		dictPaths.append("/mnt/card/opt/QtPalmtop/share/zbedic");
// 	}

  // Load the dictionaries
  QStringList activeDicts = conf.readListEntry("active-dictionaries", ':');
  loadDictionaries( activeDicts );

  descrFormatter = new DescrFormatter();
  descrFormatter->setShowPron(showPron);
  setCaption(tr("ZBEDic Dictionary"));

  // ========== create actions
  action[TB_SwitchDictionaries] = createAction( tr( "Switch dictionaries" ), "zbedic/switch_dictionary",
    SLOT(switchToPreviousDict()), CTRL+Key_S );
  action[TB_Back] = createAction( tr( "History previous" ), "zbedic/back",
    SLOT(navigationPrevious()), Key_End );
  action[TB_Forward] = createAction( tr( "History next" ), "zbedic/forward",
    SLOT(navigationNext()), Key_Home );
  action[TB_LookupSelection] = createAction( tr( "Lookup selection" ), "zbedic/lookup_selection",
    SLOT(lookupSelection()), CTRL+Key_L );
  action[TB_Copy] = createAction( tr( "Copy" ), "copy",
    SLOT(copy()) );
  action[TB_Paste] = createAction( tr( "Paste" ), "paste",    
    SLOT(paste()) ); 
  action[TB_NewEntry] = createAction( tr( "New entry" ), "new", SLOT(newEntry()), CTRL+Key_N ); 
  action[TB_EditEntry] = createAction( tr( "Edit entry" ), "edit", SLOT(editEntry()), CTRL+Key_E ); 
  action[TB_RemoveEntry] = createAction( tr( "Remove entry" ), "editdelete", SLOT(removeEntry()), CTRL+Key_Backspace ); 
  action[EX_ZoomIn] = createAction( tr( "Zoom in" ), NULL, SLOT(zoomIn()) ); 
  action[EX_ZoomOut] = createAction( tr( "Zoom out" ), NULL, SLOT(zoomOut()) ); 
  action[EX_KeyboardConfig] = createAction( tr( "Configure keyboard" ), "zbedic/custom_keys", SLOT(configureKeyboard()) ); 
  action[EX_SelectDictionary] = createAction( tr( "Select dictionary" ), NULL, SLOT(showDictionaries()), CTRL+Key_D ); 
  action[EX_ShowHistory] = createAction( tr( "Show history" ), NULL, SLOT(showHistory()), CTRL+Key_H ); 
  action[EX_ScrollDown] = createAction( tr( "Scroll down" ), NULL, SLOT(scrollLineDown()), SHIFT+Key_Down ); 
  action[EX_ScrollUp] = createAction( tr( "Scroll up" ), NULL, SLOT(scrollLineUp()), SHIFT+Key_Up ); 
//   action[EX_NextEntry] = createAction( tr( "Next entry" ), NULL, SLOT(nextEntry()), Key_Down ); 
//   action[EX_PreviousEntry] = createAction( tr( "Previous entry" ), NULL, SLOT(previousEntry()), Key_Up ); 
  action[EX_FocusOnText] = createAction( tr( "Focus on text" ), NULL, SLOT(focusOnText()), CTRL+Key_B ); 
  action[TB_Settings] = createAction( tr( "Settings" ), "SettingsIcon",    
    SLOT(settings()) );
  
  appReturnAction = new QAction( tr( "App return" ), QIconSet( Resource::loadPixmap( "zbedic/app_return" ) ),
    tr( "App return" ), 0, this, 0 );
  connect( appReturnAction, SIGNAL( activated() ), this, SLOT(appReturn()) );
  
  // ========== set up actions for keyboard shortcuts
  keyboardShortcuts = new KeyboardShortcuts( this );
  keyboardShortcuts->addAction( action[EX_SelectDictionary] );  
  keyboardShortcuts->addAction( action[TB_SwitchDictionaries] );  
  keyboardShortcuts->addAction( action[EX_ShowHistory] );
//   keyboardShortcuts->addAction( action[EX_NextEntry] );
//   keyboardShortcuts->addAction( action[EX_PreviousEntry] );
  keyboardShortcuts->addAction( action[EX_FocusOnText] );
  keyboardShortcuts->addAction( action[TB_Back] );  
  keyboardShortcuts->addAction( action[TB_Forward] );  
  keyboardShortcuts->addAction( action[EX_ScrollDown] );  
  keyboardShortcuts->addAction( action[EX_ScrollUp] );  
  keyboardShortcuts->addAction( action[TB_LookupSelection] );  
  keyboardShortcuts->addAction( action[TB_Copy] );  
  keyboardShortcuts->addAction( action[TB_Paste] );  
  keyboardShortcuts->addAction( action[TB_NewEntry] );  
  keyboardShortcuts->addAction( action[TB_EditEntry] );  
  keyboardShortcuts->addAction( action[TB_RemoveEntry] );  
  keyboardShortcuts->addAction( action[EX_ZoomIn] );  
  keyboardShortcuts->addAction( action[EX_ZoomOut] );  
  keyboardShortcuts->addAction( action[TB_Settings] );
    
  // ========== create the toolbar
  mainToolBar = new QPEToolBar(this);
  mainToolBar->setHorizontalStretchable(TRUE);
  setToolBarsMovable(FALSE);
  
  QPEMenuBar *menuBar = new QPEMenuBar( mainToolBar );
  
  wordBox = new WordBox(mainToolBar);
  CHECK_PTR(wordBox);
  wordBox->setEditable(true);
  wordBox->setInsertionPolicy(QComboBox::NoInsertion);
  wordBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred, false));

  word = wordBox->lineEdit();
//	wordBox->setFixedWidth(120);
  mainToolBar->setStretchableWidget(wordBox);
  
  // restore the values in the history
  QListBox* lb = wordBox->listBox();
  QStringList history = conf.readListEntry("history", ',');
  for(unsigned int i = 0; i < history.count(); i++) {
    int atSignPos = history[i].findRev( '@' );
    lb->insertItem(new HistoryItem( history[i].left( atSignPos ),
                     history[i].mid( atSignPos+1 ) ));          
  }
  int historyCurrent = conf.readNumEntry( "history-current", 0 );
  wordBox->setCurrentItem( historyCurrent );

  
  dictButton = new QToolButton(Resource::loadPixmap("zbedic/dictionary"), tr("Select Dictionary"),
    QString::null, this, SLOT(showDictionaries()), mainToolBar);	

  dictPopup = new QPopupMenu();
  fillInDictPopup();

  mainToolBar->addSeparator();

  // Show only action selected in config
  if( conf.hasKey("tool-bar") ) {
    QStringList toolBarConfig = conf.readListEntry("tool-bar", ';');
    for( int i = 0; i < TB_count; i++ ) {
      if( toolBarConfig.findIndex( action[i]->text() ) != -1 ) {
        action[i]->addTo( mainToolBar );
        actionInToolbar[i] = true;
      } else
        actionInToolbar[i] = false;
    }
  }  

  
  // =========== create application menu
  mainMenu = new QPopupMenu( this );  
  action[TB_SwitchDictionaries]->addTo( mainMenu );  
  
  int itemID;
//  QIconSet dictIconSet = Resource::loadIconSet("zbedic/dictionary");
  itemID = mainMenu->insertItem( Resource::loadPixmap("zbedic/dictionary"), tr("Select dictionary"), dictPopup );

  mainMenu->insertSeparator();

  action[TB_Back]->addTo( mainMenu );
  action[TB_Forward]->addTo( mainMenu );

  mainMenu->insertSeparator();

  action[TB_LookupSelection]->addTo( mainMenu );

  QPopupMenu *editMenu = new QPopupMenu( mainMenu );  

  action[TB_Copy]->addTo( editMenu );
  action[TB_Paste]->addTo( editMenu );
  
  editMenu->insertSeparator();

  action[TB_NewEntry]->addTo( editMenu );
  action[TB_EditEntry]->addTo( editMenu );
  action[TB_RemoveEntry]->addTo( editMenu );

  mainMenu->insertItem( tr("Edit"), editMenu );
  
  mainMenu->insertSeparator();

  itemID = mainMenu->insertItem( tr("Zoom in") );
  mainMenu->connectItem( itemID, this, SLOT(zoomIn()) );
  itemID = mainMenu->insertItem( tr("Zoom out") );
  mainMenu->connectItem( itemID, this, SLOT(zoomOut()) );
  
  mainMenu->insertSeparator();

  action[EX_KeyboardConfig]->addTo( mainMenu );
  action[TB_Settings]->addTo( mainMenu );

  QPixmap menuPixmap = Resource::loadPixmap("zbedic/menu");
  menuBar->insertItem( menuPixmap, mainMenu );
  menuBar->setMaximumWidth( menuPixmap.width() + 8 ); //Fixes QT bug (menu bar takes full window width)

  //=================
  
  QWidget* w = new QWidget(this);
  
  setCentralWidget(w);
  mainLayout = new QBoxLayout(w, QBoxLayout::TopToBottom);
  mainLayout->setSpacing(4);
  mainLayout->setMargin(2);

  // Create and set list (of words)
  wordList = new WordList(w);
  CHECK_PTR(wordList);
  wordList->setHScrollBarMode(QScrollView::AlwaysOff );
  wordList->setFocusPolicy(QWidget::NoFocus);
  wordList->setSelectionMode( QListBox::Single );

//  wordList->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
  mainLayout->addWidget(wordList, 1);

  // The description box and the editor are put in a WidgetStack
  viewEditStack = new NoSizeWidgetStack( w );
  viewEditStack->setMinimumSize( QSize( 0, 0 ) );
  viewEditStack->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred ) );
  
  descr = new DictBrowser(viewEditStack);
  CHECK_PTR(descr);
  descr->setTextFormat(QTextView::RichText);
  descr->setFocusPolicy( QWidget::ClickFocus );
  viewEditStack->addWidget( descr, DISP_VIEW );

  zbedicEditor = new ZBEDicEditor( viewEditStack );
  viewEditStack->addWidget( zbedicEditor, DISP_EDIT );
  viewEditStack->raiseWidget( DISP_VIEW );
  connect(zbedicEditor, SIGNAL(editDone(QString)), 
    this, SLOT(finishEditingOK(QString)));
  connect(zbedicEditor, SIGNAL(editCanceled()), 
    this, SLOT(finishEditingCancel()));
  
  mainLayout->addWidget(viewEditStack, 2);

  updateLayout( wordListSize, layoutMode );

  // restore current dictionary
  QString dname = conf.readEntry("dictionary", QString::null);
  StaticDictionary* d = findDictionary(dname);
  if (d == 0) {
    if (dictionaries.count() > 0) {
      d = dictionaries.at(0);
    }
  }

  setDictionary(d);

  // TODO: Fixme: Nasty trick to make the size of word list OK
  word->clear();
  wordList->clear();        
		
  // get application clipboard
  systemClipboard = QApplication::clipboard();

  word->setFocus();

  if (dict!= 0) {
    // restore the state
    QString w = conf.readEntry("word");
    updateWordLookup( w, true, true );
//    word->setText( w );
//    wordChanged( w );
  }

  addToHistoryTimer = new QTimer( this );
  connect(addToHistoryTimer, SIGNAL(timeout()), 
    this, SLOT(addToHistoryTriggered()));

        
  //Accelerators
  QAccel *accel = new QAccel( this );  
//   accel->connectItem( accel->insertItem( '#' + CTRL ),
//     this, SLOT( showDictionaries( void ) ) );
//   accel->connectItem( accel->insertItem( '_' + CTRL ),
//     wordBox, SLOT( activatePopup( void ) ) );
//   accel->connectItem( accel->insertItem( '@' + CTRL ),
//     this, SLOT( switchToPreviousDict() ) );
//   accel->connectItem( accel->insertItem( '-' + CTRL ),
//     descr, SLOT( setFocus() ) );

  // SL-C700
//   accel->connectItem( accel->insertItem( 0x1021 + CTRL ),
//     this, SLOT( showDictionaries( void ) ) );
//   accel->connectItem( accel->insertItem( 0xa5 + CTRL + SHIFT ),
//     this, SLOT( showDictionaries( void ) ) );
//   accel->connectItem( accel->insertItem( 0x2018 + CTRL + SHIFT ),
//     this, SLOT( switchToPreviousDict() ) );
//   accel->connectItem( accel->insertItem( 0x3a + CTRL + SHIFT ),
//     wordBox, SLOT( activatePopup( void ) ) );
//   accel->connectItem( accel->insertItem( 0x5f + CTRL + SHIFT ),
//     descr, SLOT( setFocus() ) );
  accel->connectItem( accel->insertItem( 0x2010 ),
    this, SLOT( zoomIn() ) );
  accel->connectItem( accel->insertItem( 0x200f ),
    this, SLOT( zoomOut() ) );
//   accel->connectItem( accel->insertItem( Key_F11 ),
//     this, SLOT( showMenu() ) );  
    
  connect(word, SIGNAL(textChanged(const QString&)), 
    this, SLOT(wordChanged(const QString&)));

//   connect(wordBox, SIGNAL(activated(int)), 
//     this, SLOT(historyItemActivated(int)));
  
  connect(dictPopup, SIGNAL(activated(int)), 
    this, SLOT(selectDictionary(int)));

  connect(wordList, SIGNAL(highlighted(const QString&)), 
    this, SLOT(listPick(const QString&)));

  connect(descr, SIGNAL(anchor(const QString&)), 
    this, SLOT(seeAlso(const QString&)));

  adjustSize();

  QObject::connect(qApp, SIGNAL(appMessage(const QCString&, const QByteArray&)), 
    this, SLOT(msgHandler(const QCString&, const QByteArray&)));

  // Not used at the moment
  
//   QCopChannel* c1 = new QCopChannel("QPE/Card", this);
//   QObject::connect(c1, SIGNAL(received(const QCString&, const QByteArray&)),
//     this, SLOT(c1Handler(const QCString&, const QByteArray&)));
  
//   QCopChannel* c2 = new QCopChannel("QPE/System", this);
//   QObject::connect(c2, SIGNAL(received(const QCString&, const QByteArray&)),
//     this, SLOT(c2Handler(const QCString&, const QByteArray&)));

  keyboardShortcuts->load( &conf );
  
  fullyInitialized = true;
}


// void ZBEDic::polish()
// {
//   QMainWindow::polish();

//   Config conf("zbedic");
//   conf.setGroup("general");

//   QString w = conf.readEntry("word");
//   word->setText( w );
//   wordChanged( w );
  
// }

QAction *ZBEDic::createAction( const QString &name, const char *icon, const char *slot, int defaultKey )
{
  QAction *a;
  if( icon == NULL ) {
    a = new QAction( name, name, 0, this, 0 );
  } else {
    a = new QAction( name,
      QIconSet( Resource::loadPixmap( icon ) ),
      name, 0, this, 0 );
  }
  connect( a, SIGNAL( activated() ),
    this, slot );
  if( defaultKey != -1 )
    a->setAccel( defaultKey );
  return a;
}

void ZBEDic::closeEvent( QCloseEvent * e )
{
  // Save config and exit
  Config conf("zbedic");
  conf.setGroup("general");

  conf.writeEntry("word", word->text());

  // save the history
  QStringList history;
  QListBox* list = wordBox->listBox();
  for(unsigned int i = 0; i < list->count(); i++) {
    HistoryItem *item = (HistoryItem*)list->item( i );
    const QString &itemDict = item->getDictionary();
    history.append(list->text(i) + "@" + itemDict );
  }
  conf.writeEntry("history", history, ',');
  conf.writeEntry("history-current", wordBox->currentItem() );
        
  QStringList activeDicts;
  for(unsigned int i = 0; i < dictionaries.count(); i++) {
    activeDicts.append( dictionaries.at(i)->getFileName() );
  }
  conf.writeEntry("active-dictionaries", activeDicts, ':');

  conf.writeEntry("layout-mode", layoutMode);
  conf.writeEntry("word-list-size", wordListSize);
  conf.writeEntry("history-length", (int) historyLen);
  conf.writeEntry("list-length", (int) listLen);
  conf.writeEntry("previous-dictionary", previousDict);
//	conf.writeEntry("dict-path", dictPaths, ':');

  conf.writeEntry("search-dict-loc", searchInLocation );
  conf.writeEntry("search-dict-all", searchInAllDocs );

  if (dict != 0) {
    conf.writeEntry("dictionary", QString(dict->getName()));
  }

  conf.writeEntry("font-name", fontName);
  conf.writeEntry("font-size", fontSize);

  conf.writeEntry("show-pron", showPron);

  QStringList toolBarConfig;
  for( int i = 0; i < TB_count; i++ ) {
    if( actionInToolbar[i] )
      toolBarConfig.append(action[i]->text());
  }
  conf.writeEntry("tool-bar", toolBarConfig, ';');

  keyboardShortcuts->save( &conf );

  QMainWindow::closeEvent( e );
}

ZBEDic::~ZBEDic() {
        
  delete descrFormatter;
  delete dictPopup;

  for(unsigned int i = 0; i < dictionaries.count(); i++) {
    delete dictionaries.at(i);
  }
}

void ZBEDic::wordChanged(const QString& text)
{
  if( onWordLookup ) return;

//  saveScrollPosition();

//  printf( "Word changed %s\n", text.latin1() );

  if( isCurrentHistoryItem( text ) ) { // If selected from the history
    HistoryItem *item = getCurrentHistoryItem();
//    printf( "Is history item: %s\n", item->text().latin1() );

//    descr->viewport()->setUpdatesEnabled( FALSE );
    
    StaticDictionary *itemDictionary = findDictionary( item->getDictionary() );
    if( itemDictionary != NULL && itemDictionary != dict ) {
      setDictionary( itemDictionary );
    } else {
      updateWordLookup( text, TRUE, FALSE );
    }      

    // Those three calls seems to be the only way to make scrolling in
    // large entries work on SL5500
    qApp->processEvents();
    qApp->processEvents();
    qApp->processEvents();

//    printf( "Height after update: %d\n", descr->contentsHeight() );

    descr->setContentsPos( descr->contentsX(), item->getScrollPosition() );

//     printf( "Item: %s Going to pos: %d, achieved: %d\n", item->text().latin1(),
//       item->getScrollPosition(), descr->contentsY()  );
    
//    descr->viewport()->setUpdatesEnabled( TRUE );
    
    return;
  }
  
  if( updateWordLookup( text, TRUE, FALSE ) )
    addToHistoryDelayed( text );
}

void ZBEDic::listPick(const QString& text)
{
  if( onWordLookup ) return;

  saveScrollPosition();
  
  updateWordLookup( text, FALSE, TRUE );
  addToHistoryDelayed( text );

  word->selectAll();
  word->setFocus();
}

// =============== Lookup for a word ================

bool ZBEDic::updateWordLookup( const QString &newWord, bool updateWordList, bool updateWordBox )
{
  onWordLookup = true;
    
  bool exactMatch = false;
  QString w;
  QString m;
  
  if( updateWordBox )
    word->setText(newWord);
  
  // Look up dictionary
  if (dict != 0) {
    DictionaryIteratorPtr iter = newWord.isEmpty() ? dict->begin() : dict->findEntry( newWord.utf8(), exactMatch );

    if( newWord.isEmpty() ) {
      exactMatch = false;
    }
    if( !iter.isValid() )
      goto lookup_error;

    
    if( iter != dict->end() ) {
      const char *str = iter->getKeyword();
      if( str == NULL )
        goto lookup_error;
      w = QString::fromUtf8(str);
      str = iter->getDescription();
      if( str == NULL )
        goto lookup_error;
      m = QString::fromUtf8(str);
    }
    
    // Set description if exact match
    QString formattedDescription;
    if( exactMatch ) {
      formattedDescription = descrFormatter->format(m, w);
      descrWord = w;              // newWord and descrWord may differ due to canonization
      descrText = formattedDescription;
      descr->setText(descrText);
    } else {
      descrWord = QString::null;
      descrText = QString::null;
      if( newWord.isEmpty() ) 
        descr->setText( getDictFrontpage( dict ) );
      else
        descr->setText(descrText);
    }

    
    // Fill out the list of words if update needed
    if( updateWordList &&
      (wordList->count()==0 || wordList->text(0) != w) &&
      dict != 0 ) {
      
      wordList->clear();
          
      unsigned int i = 0;
      while( iter != dict->end() && (i < listLen)) {  
        wordList->insertItem(QString::fromUtf8(iter->getKeyword()));
        i++;
        iter->nextEntry();
      } 
      wordList->setTopItem(0);                        
    }

    // Select a word from the list if exact match
    if( updateWordList ) {
      if( exactMatch )
        wordList->setSelected( 0, true ); // exact match must be 1st item
      else { // Deselect
        int item = wordList->currentItem();
//        if( item != -1 ) wordList->setSelected( item, false );
        if( item != -1 ) {
          wordList->clearSelection();          
        }
        
      }
    }
  }

  updateNavigationButtons();

  //Update actions (enable/disable)
  {
    bool dictionaryPresent = dict != NULL;
    action[TB_NewEntry]->setEnabled( dictionaryPresent && !exactMatch );
    action[TB_EditEntry]->setEnabled( dictionaryPresent && exactMatch );
    action[TB_RemoveEntry]->setEnabled( dictionaryPresent && exactMatch );
  }
  
  
  onWordLookup = false;
  return exactMatch;
  
lookup_error:

  descr->setText( QString( tr( "Database error:" ) ) + dict->getErrorMessage() );

  return false;
  
}

void ZBEDic::copy()
{
  if (descr->hasSelectedText()) {
    descr->copy();
  } else if (word->hasMarkedText()) {
    word->copy();
  }
}

void ZBEDic::paste() {
  word->setFocus();
  word->clear();
//	dict->randomEntry();
//	word->setText(QString::fromUtf8(dict->getWord().c_str()));
  word->paste();
}

// Return to calling application after receiving the 'find' message
// (e.g. from OpieReader)
void ZBEDic::appReturn() {
  hide();
  appReturnAction->removeFrom( mainToolBar );
//  lower();
//  settingsButton->show();
}


bool ZBEDic::event( QEvent * e )
{
  // Remove 'Return to app' button when the window is deacivated
  if( e->type() == QEvent::WindowDeactivate )
    appReturnAction->removeFrom( mainToolBar );

  return QMainWindow::event( e );
}

void ZBEDic::selectDictionary(int i) {
  StaticDictionary *d;
  if( dictionaries.count() == 0 )
    d = NULL;
  else
    d = dictionaries.at(i);

  setDictionary( d );        
}

void ZBEDic::showMenu()
{
  mainMenu->exec();
}


StaticDictionary* ZBEDic::findDictionary(const QString& name) {
  unsigned int i;
  StaticDictionary* d = 0;

  for(i = 0; i < dictionaries.count(); i++) {
    if (name == dictionaries.at(i)->getName()) {
      d = dictionaries.at(i);
      break;
    }
  }
	
  return d;
}

void ZBEDic::setDictionary(StaticDictionary* d)
{
  if( dict != NULL )
    previousDict = dict->getName();

  dict = d;
  for(uint i = 0; i < dictionaries.count(); i++) {
    dictPopup->setItemChecked(i, d == dictionaries.at(i));
  }

  if (dict != 0) {
    setCaption("zbedic: " + QString(dict->getName()));
    if( updateWordLookup( word->text(), TRUE, FALSE ) )
      addToHistoryDelayed( word->text() );
  } else {
    wordList->clear();
    descr->setText("<qml><p>No dictionaries found. The reason may be:</p>"
      "<ul>"
      "<li>Dictionaries are installed, but not activated. Go to the settings dialog, "
      "Dictionaries tab, search for new dictionaries and make active the dictionaries "
      "you want to use. You can also open dictionary files from Document Manager to "
      "activate them.</li>"
      "<li>No dictionaries were installed. You can download "
      "dictionaries from "
      "http://www.sourceforge.net/projects/bedic</li>");
  }	

}

void ZBEDic::showDictionaries() {
  QPoint p = dictButton->mapToGlobal(QPoint(0, dictButton->height()));
  dictPopup->exec(p);
}

void ZBEDic::seeAlso(const QString& text) {
  findWord(text);
}

void ZBEDic::findWord(const QString& text)
{
  addToHistoryTriggered();      // Allow for back navigation
  
  saveScrollPosition();

  if( updateWordLookup( text ) )
    addToHistory( text );
}


// ============= History ===========
bool ZBEDic::isCurrentHistoryItem( const QString &word, StaticDictionary *dic )
{ 
  HistoryItem *item = getCurrentHistoryItem();
  if( item == NULL ) return false;

//   if( dic != NULL )
//     printf( "'%s' '%s'\n", dic->getName().c_str(), item->getDictionary().latin1() );
  
  if( dic != NULL && dic->getName() != item->getDictionary().latin1() )
    return false;

//  printf( "Current hist: '%s',  word: '%s'\n", item->text().latin1(), word.latin1() );

  return item->text() == word;
}

HistoryItem *ZBEDic::getCurrentHistoryItem()
{
  if( wordBox->currentItem() < 0 || wordBox->currentItem() >= (int)wordBox->count() ) return NULL;
  return (HistoryItem*)wordBox->listBox()->item( wordBox->currentItem() );
}

void ZBEDic::saveScrollPosition()
{
  // For backward navigation: if previous word is in history, save scroll position
  // of the description box
  if( isCurrentHistoryItem( word->text() ) ) {
    getCurrentHistoryItem()->setScrollPosition( descr->contentsY() );
//    printf( "Scroll pos saved: %d\n", descr->contentsY() );
  }
}

// void ZBEDic::historyItemActivated(int idx)
// {
//    printf( "hist item activ %d\n", idx );
// //   QListBox* list = wordBox->listBox();
// //   HistoryItem *item = (HistoryItem*)list->item( idx );
// //   gotoHistoryItem( item );   
// }


void ZBEDic::addToHistoryTriggered()
{
  addToHistoryTimer->stop();
  if( wordToAddToHistory.compare( wordBox->currentText() ) != 0 ) return;
  addToHistory( wordToAddToHistory );
  wordToAddToHistory = QString::null;  
}


void ZBEDic::addToHistory(const QString& s) {
  QListBox* list = wordBox->listBox();
  
//  printf( "Add to hist\n" );

//   for(unsigned int i = 0; i < list->count(); i++) {
//     HistoryItem* item = (HistoryItem*)list->item(i);
//     if (s == item->text()) {
//       return;
//     }
//   }

  HistoryItem *newItem = new HistoryItem( s, dict->getName() );

  int insertAt = wordBox->currentItem();
  if( insertAt == -1 ) insertAt = 0;

  if( getCurrentHistoryItem() != NULL ) {
    // Special case - don't add to history if current or next entry is
    // the same as the new entry
    if( getCurrentHistoryItem()->text() == s ) return;

    if( insertAt > 0 && wordBox->listBox()->item( insertAt-1 )->text() == s ) {
      //TODO: problem may appear if dictionary is different
      wordBox->setCurrentItem( wordBox->currentItem()-1 );
      return;
    }
  }
  
  list->insertItem( newItem, insertAt);

  // Remove some items if more then historyLen
  while(list->count() > historyLen)
  {
    //Delete item either from the beginning or end of the history
    if( wordBox->currentItem() < (int)list->count()/2 )
      list->removeItem(list->count() - 1);
    else {
      list->removeItem(0);
      wordBox->setCurrentItem( wordBox->currentItem()-1 );
    }
  }

  updateNavigationButtons();
}


void ZBEDic::addToHistoryDelayed( const QString& word )
{  
  // If exact match, activate 3s timer, which adds the word to the history
  // fullyInitialized is needed because the timer should not be started from the
  // constructor
  if( fullyInitialized ) {
    wordToAddToHistory = word;
    addToHistoryTimer->start( 3000, true );
  }
  
}

// ============= Back / Forward navigation  ===========
void ZBEDic::updateNavigationButtons()
{
  int historyItem = wordBox->currentItem();
  bool isValid = historyItem < (int)wordBox->count() && historyItem != -1;
    
  action[TB_Back]->setEnabled( isValid &&
    (!isCurrentHistoryItem( word->text(), dict ) || historyItem != wordBox->count()-1) );
  action[TB_Forward]->setEnabled( isValid && historyItem > 0 );
}

  

void ZBEDic::navigationNext()
{
  saveScrollPosition();
  if( wordBox->currentItem() > 0 && wordBox->currentItem() < (int)wordBox->count()) {
    saveScrollPosition();
    wordBox->setCurrentItem( wordBox->currentItem()-1 );
  }
}

void ZBEDic::navigationPrevious()
{
  if( !isCurrentHistoryItem( word->text(), dict ) &&
    wordBox->currentItem() < (int)wordBox->count() ) { // If between history entries
    saveScrollPosition();
    wordBox->setCurrentItem( wordBox->currentItem() );
  } else if( wordBox->currentItem() < (int)wordBox->count()-1 ) {
    saveScrollPosition();
    wordBox->setCurrentItem( wordBox->currentItem()+1 );
  }
}

void ZBEDic::loadDictionaries(const QStringList& activeDicts)
{
  for( uint i = 0; i < activeDicts.count(); i++ ) {
    loadSingleDictionary( activeDicts[i], false );
  }
}

StaticDictionary *ZBEDic::loadSingleDictionary(const QString &fileName, bool checkIntegrity)
{
  string errorMsg;
  StaticDictionary* d = StaticDictionary::loadDictionary( fileName, checkIntegrity, errorMsg );
  
  static bool deactivateAllFiles = false;
    
  if( d == NULL ) {
    if( deactivateAllFiles == false ) {
      int n = fileName.findRev('/');
      QString file = fileName.mid(n+1);
      QString path = fileName.left(n);
      QString ss = "File: " + file + "\nPath: " + path + 
        "\nError: " + errorMsg.c_str() +
        "\nDeactivate the dictionary file?";
      int res = QMessageBox::warning(0, "zbedic: Error", ss, "Yes", "Yes to all", "Quit");
      if( res == 1 ) 
        deactivateAllFiles = true;
      else if( res == 2 )
        exit( 1 );
    }
    return NULL;
  } else {
    dictionaries.inSort(d);
    return d;
  }
}

void ZBEDic::updateFont(QFont &font) {
  fontHeight = QFontMetrics( font ).height();
  setFont( font );
}

void ZBEDic::changeFont(QString& fname, int fsize) {
	fontName = fname;
	fontSize = fsize;

	QSize dsize = descr->size();
	QSize lsize = wordList->size();

	descr->setFixedSize(dsize);
	wordList->setFixedSize(lsize);
	
	font = QFont(fontName, fontSize);
	updateFont(font);
	descr->setFont(font);
	update();

	descr->setText(descrText);
}

void ZBEDic::msgFind(QString word) {
	if (!word.isEmpty()) {
		findWord(word);
	}
}

void ZBEDic::msgFind(QString dictionary, QString word) {
	if (dictionary.isEmpty()) {
		return;
	}

	StaticDictionary* d = findDictionary(dictionary);

	if (d == NULL) {
		return;
	}

	setDictionary(d);
	msgFind(word);
}

void ZBEDic::msgHandler(const QCString& msg, const QByteArray& data) {
  
  QDataStream stream(data, IO_ReadOnly);

//  QMessageBox::warning(this, msg, data.data() );
//  printf("msgHandler: %s\n", (const char*) msg);

  bool messageRecognized = false;

  if( msg == "find(QString)" ) {
    QString word;

    stream >> word;
    msgFind(word);
    messageRecognized = true;
                
  } else if( msg == "find(QString,QString)" ) {
    QString dictionary;
    QString word;

    stream >> dictionary;
    stream >> word;
    msgFind(dictionary, word);
    messageRecognized = true;
  } else if( msg == "nextView()" ) {
    if( descr->hasFocus() ) 
      word->setFocus();
    else
      descr->setFocus();
  }
  

  if( messageRecognized ) {
    // To make sure only one button is displayed
    appReturnAction->removeFrom( mainToolBar );
    appReturnAction->addTo( mainToolBar );
    show();
    raise();
    ((QPEApplication*)qApp)->setKeepRunning();
  }
}

// Not used at the moment
// 
// void ZBEDic::c1Handler(const QCString& msg, const QByteArray& data) {
// 	printf("c1Handler: %s\n", (const char*) msg);
// }

// void ZBEDic::c2Handler(const QCString& msg, const QByteArray& data) {
// 	printf("c2Handler: %s\n", (const char*) msg);
// }


DictBrowser::DictBrowser(QWidget* parent, const char* name) : 
  QTextBrowser(parent, name)
{
  originalFrameStyle = frameStyle();
}

void DictBrowser::setSource(const QString& name) {
	emit anchor(name);
}

void DictBrowser::focusInEvent(QFocusEvent *e)
{
  QTextBrowser::focusInEvent( e );
  setFrameStyle( QFrame::Panel + QFrame::Plain );
  update();
}

void DictBrowser::focusOutEvent(QFocusEvent *e)
{
  QTextBrowser::focusOutEvent( e );
  setFrameStyle( originalFrameStyle );  
  update();
}

DescrFormatter::DescrFormatter() {
	showPron = false;
}

QString DescrFormatter::format(const QString& text, const QString& headword) {
  QString result;
  QString etag;

  headWord = headword;
  partOfSpeech = QString();
  pron = QString();
  ndef = 0;

  if (processBody(text, 0, etag, result) >= 0) {
//          printf( "===============\n%s===============\n", result.latin1() );
    
    // Process escape character 
    int i = 0;
    while( true )
    {
      i = result.find( '\\', i );
      if( i < 0 ) break;
      result.remove( i, 1 );
    }
          
    return result;
  } else {
    return text;
  }
}

static int findNonEscaped( const QString &s, const char c, int from )
{
  int i;
  while( true )
  {
    i = s.find( c, from );
    if( i < 0 ) return i;
    if( i == 0 || s[i-1] != '\\' )
      return i;
    from = i+1;
  }
}


int DescrFormatter::processBody(const QString& text, int idx, QString& etag, 
	QString& ret) {

  int n, i = idx;
  QString savePartOfSpeech;
  
  while ((n = findNonEscaped( text, '{', i)) >= 0) {
    
    // append the body text to the result
    ret += text.mid(i, n - i);

    // find the end of the tag
    int k = findNonEscaped( text, '}', n+1 );
    
    if (k < 0) {
      return -1;
    }

    // get the tag and whether it is empty
    QString tag = text.mid(n+1, k-n-1);
    bool empty = false;

    if (tag[tag.length()-1] == '/') {
      empty = true;
      tag = tag.left(tag.length() - 1);
    }

    // if the tag is closing tag, check if it is
    // the one we are looking for
    if (tag[0] == '/') {
      tag = tag.right(tag.length() - 1);
      if (tag != etag) {
        // non-matching end tag -- error
        return -1;
      } else {
        // found it, return
        return k+1;
      }
    }
    i = k+1;

    if (tag == "s") {
      savePartOfSpeech = partOfSpeech;
      ndef=0;
    }

    QString val;

    if (!empty) {
      QString ctag;
      int space = tag.find( ' ' );
      if( space > 0 ) {
        ctag = tag.left( space );
//        printf( "tag: %s\n", ctag.latin1() );
      } else
        ctag = tag;
      i = processBody(text, i, ctag, val);
    }

    if (i < 0) {
      // ignore the tag
      i = k+1;
      continue;
    }

    if (tag == "hw") {
      if( empty ) 
        ret += "<b>" + headWord + "</b>";
      else {
        headWord = val;
      }
                  
    } else if (tag == "ps") {
      partOfSpeech = val;
    } else if (tag == "s") {
      ret += "<b>" + headWord + "</b> ";
      if (!pron.isEmpty() && showPron) {
        ret += " <font color=#600070>/" + pron + "/</font>";
      }

      if (!partOfSpeech.isEmpty()) {
        ret += " <i><font color=#007070>" + partOfSpeech + "</font></i>";
      }

      if (!category.isEmpty()) {
        ret += " <font color=#707040>" + category + "</font>";
        category = QString();
      }
                        
      if (ndef > 1) {
        ret += "<ol>" + val + "</ol><hr/>";
      } else {
        while ((k = val.find("<li>")) >= 0) {
          val.remove(k, 4);
        }

        while ((k = val.find("</li>")) >= 0) {
          val.remove(k, 5);
        }

        ret += val + "<hr/>";
      }
      partOfSpeech = savePartOfSpeech;
    } else if (tag == "ss") {
      int ti, tn;
      ret += "<li>";
      for(ti = 0; (tn = val.find('\n', ti)) >= 0; ti = tn+1) {
        ret += "<p>" + val.mid(ti, tn-ti) + "</p>";
      }

      ret += "<p>" + val.right(val.length() - ti) + "</p></li>";
      ndef++;
    } else if (tag == "ex") {
      if( etag == "ss" )
        ret += "<i><font color=#006000>" + val + "</font></i>";
      else
        ret += "<p><font color=#006060>" + val + "</font></p>";
    } else if (tag == "ct") {
      if( etag == "ss" )
        ret += "<font color=#707040>" + val + "</font>";                  
      else
        category = val;
    } else if (tag == "em") {
      ret += "<b>" + val + "</b>";                  
    } else if (tag == "de") {
      ret += "<font color=#606060>" + val + "</font>";                  
    } else if (tag.left(2) == "sa") {
      if( tag == "sa" ) 
        ret += "<a href=\"" + val + "\">" + val + "</a>";
      else
        ret += "<a href=\"" + tag.mid(3) + "\">" + val + "</a>";
    } else if (tag == "pr") {
      pron = val;
    } else if (tag == "br") {
      ret += "<br/>";
    } else {
      return -1;
    }
  }

  ret += text.right(text.length() - i);

  return text.length();
}

void ZBEDic::keyPressEvent ( QKeyEvent * e )
{
  if( e->key() == Key_Down && e->state() != ShiftButton) 
    nextEntry();
  else if( e->key() == Key_Up && e->state() != ShiftButton)
    previousEntry();
  else
    QMainWindow::keyPressEvent( e );
}


void ZBEDic::updateLayout( int wordListSize, int layoutMode )
{
  int stretchFactor;
  if( wordListSize == 0 ) stretchFactor = 4;
  else if( wordListSize == 1 ) stretchFactor = 2;
  else stretchFactor = 1;
  
  mainLayout->setStretchFactor( viewEditStack, stretchFactor );

  QBoxLayout::Direction dir;
  if( layoutMode == 1 ) dir = QBoxLayout::TopToBottom;
  else if( layoutMode == 2 ) dir = QBoxLayout::LeftToRight;
  else {
    dir = QApplication::desktop()->width() > QApplication::desktop()->height() ?
      QBoxLayout::LeftToRight : QBoxLayout::TopToBottom;
  }

  mainLayout->setDirection( dir );  
}

void ZBEDic::fillInDictPopup()
{
  dictPopup->clear();
  for(unsigned int i = 0; i < dictionaries.count(); i++) {
    StaticDictionary* d = dictionaries.at(i);
    dictPopup->insertItem(QString(d->getName()), i, i);
    if( d == dict )
      dictPopup->setItemChecked(i, true);
  }
}

void ZBEDic::switchToPreviousDict()
{
  if( previousDict != QString::null ) {
    StaticDictionary *d = findDictionary( previousDict );
    if( d != NULL )
      setDictionary( d );
  }
}

          
void ZBEDic::setDocument(const QString& fileref)
{
    if ( fileref.isNull() )
	return;

    //If the dictionary is available, change to that dict
    for(unsigned int i = 0; i < dictionaries.count(); i++) {
      if( fileref == dictionaries.at(i)->getFileName() ) {
        setDictionary( dictionaries.at(i) );
        return;
      }
    }

    if( QMessageBox::information( this, "zbedic",
          "Should the selected dictionary be\n"
          "added to the list of active\n"
          "dictionaries? ",
          "Add", "Cancel", QString::null,
          0,      
          1 ) == 0 ) {

      StaticDictionary *d = loadSingleDictionary( fileref, true );
      if( d != NULL )
        setDictionary( d );
    }    
}

void WordBox::keyPressEvent ( QKeyEvent * e )
{
  // Ignore up / down keys to handle them in main window
//  printf( "Key press event\n" );
  if( e->key() == Key_Down || e->key() == Key_Up ) e->ignore();
  else if( e->key() == Key_Tab && e->state() == NoButton ) lineEdit()->selectAll();
  else QComboBox::keyPressEvent ( e );
}  

void ZBEDic::lookupSelection()
{
  if( !descr->hasSelectedText() ) {
    Global::statusMessage ( tr( "Select the text first!" ) );
    return;
  }

  addToHistoryTriggered();      // Allow for back navigation  
  saveScrollPosition();
  
  QString selection = descr->selectedText().stripWhiteSpace();
  if( updateWordLookup( selection, true, true ) )
    addToHistory( selection );
}

void ZBEDic::resizeEvent( QResizeEvent *e )
{
//  printf( "Resize event!\n" );
  if( layoutMode == 0 && fullyInitialized )
    updateLayout( wordListSize, layoutMode );
  QMainWindow::resizeEvent( e );
}

void ZBEDic::configureKeyboard()
{
  keyboardShortcuts->execConfig();
}

void ZBEDic::showHistory()
{
  wordBox->activatePopup();
}

void ZBEDic::scrollLineDown()
{
  descr->scrollBy( 0, fontHeight );  
}

void ZBEDic::scrollLineUp()
{
  descr->scrollBy( 0, -fontHeight );
}

void ZBEDic::nextEntry()
{
  // Select a next item from the word list
  int item = wordList->currentItem();
//  printf( "item: %d\n", item );
  // To fix QT bug
  if( item != -1 && !wordList->isSelected( item ) )
    item = -1;
  item++;
//  printf( "item to sel: %d; wl c %d\n", item, (int)wordList->count() );
  if( item < (int)wordList->count() ) {
    wordList->setCurrentItem( item );
  }
}

void ZBEDic::previousEntry()
{
  // Select a previous item from the word list
  int item = wordList->currentItem();
  if( item != 0 ) {
    item--;
    if( item < 0 ) item = 0;
    wordList->setCurrentItem( item );
  }
}

void ZBEDic::focusOnText()
{
  descr->setFocus();  
}

void ZBEDic::showDictionaryError()
{
  if( dict == NULL ) return;
  QMessageBox::warning(0, "zbedic: Dictionary Error", QString("Dictionary error:\n") + dict->getErrorMessage() );
}

bool ZBEDic::createHybridDic()
{
  if( !dict->isDynamic() ) {
    int res = QMessageBox::information(this, "zbedic", tr("Dictionary '") + dict->getName() +
      tr("'\nis not editable.\nShould I convert it to\nan editable dictionary?"),
      QMessageBox::Yes, QMessageBox::No );
    if( res == QMessageBox::No ) return false;

    QString hybrid_file_name = dict->getFileName();
    if( hybrid_file_name.right( 7 ) != ".dic.dz" )
      return false;
    hybrid_file_name.replace( hybrid_file_name.length() - 7, 7, ".hdic" );

    std::string errorMessage;
    DynamicDictionary *hybrid_dic = createHybridDictionary( hybrid_file_name, dict, errorMessage );

    if( hybrid_dic == NULL ) {
      QMessageBox::warning(this, "zbedic: Dictionary Error", QString("Dictionary error:\n") + errorMessage.c_str() );
      return false;
    }

    for(unsigned int i = 0; i < dictionaries.count(); i++) {
      if( dictionaries.at(i) == dict ) {
        dictionaries.insert(i, hybrid_dic );
        dictionaries.remove(i+1);
        break;
      }
    }
    dict = hybrid_dic;
  }

  return true;
}

void ZBEDic::newEntry()
{
  // Something is really wrong with QString::utf8() in QT
  string new_word( word->text().utf8() );

  if( dict == NULL || word->text().length() == 0 ) return;

  // Do not add word if already exists
  bool exactMatch;              
  dict->findEntry( new_word.c_str(), exactMatch );
  if( exactMatch ) return;  

  createHybridDic();
  
  if( dict->isDynamic() ) {
    DynamicDictionary *d = (DynamicDictionary*)dict;
    DictionaryIteratorPtr iter = d->insertEntry( new_word.c_str() );
    if( !iter.isValid() ) {
      showDictionaryError();
      return;
    }
    updateWordLookup( word->text(), true, false );
    editEntry();
  }  
}

void ZBEDic::editEntry()
{
  if( dict == NULL ) return;
  
  string keyword( word->text().utf8() );

  // Must exist in the dictionary to remove
  bool exactMatch;
  DictionaryIteratorPtr iter = dict->findEntry( keyword.c_str(), exactMatch );
  if( !iter.isValid() )
    showDictionaryError();    
  if( !exactMatch ) return;

  createHybridDic();
  
  const char *description = iter->getDescription();
  if( description == NULL )
    showDictionaryError();

  viewEditStack->raiseWidget( DISP_EDIT );
  zbedicEditor->startEditing( word->text(), QString::fromUtf8(formatDicEntry(description).c_str()) );
}

void ZBEDic::finishEditingOK( QString new_text )
{
  if( dict == NULL ) return;
  
  // Must exist in the dictionary to edit
  bool exactMatch;
  DictionaryIteratorPtr iter = dict->findEntry( zbedicEditor->getKeyword().utf8(), exactMatch );
  if( !exactMatch ) return;

  if( dict->isDynamic() ) {
    DynamicDictionary *d = (DynamicDictionary*)dict;
    bool success = d->updateEntry( iter, new_text.utf8() );
    if( !success )
      showDictionaryError();
    updateWordLookup( zbedicEditor->getKeyword(), true, false );
  }  

  viewEditStack->raiseWidget( DISP_VIEW );
  word->setFocus();
}

void ZBEDic::finishEditingCancel( )
{
  viewEditStack->raiseWidget( DISP_VIEW );
  word->setFocus();
}

void ZBEDic::removeEntry()
{
  if( dict == NULL ) return;
  
  string keyword( word->text().utf8() );

  // Must exist in the dictionary to remove
  bool exactMatch;
  DictionaryIteratorPtr iter = dict->findEntry( keyword.c_str(), exactMatch );
  if( !exactMatch ) return;
  
  int res = QMessageBox::information(0, "zbedic",
    tr("Remove entry '") + word->text() + tr("'?"), tr("Remove"), tr("Cancel") );

  if( res == 1 ) return;
  
  if( dict->isDynamic() ) {
    DynamicDictionary *d = (DynamicDictionary*)dict;
    bool success = d->removeEntry( iter );
    if( !success )
      showDictionaryError();
    updateWordLookup( word->text(), true, false );
  }
  
}


static QString escapeString( const string &str )
{
  QString s = QString::fromUtf8( str.c_str() );
  s = s.replace( QRegExp("<"), "&lt;" );
  s = s.replace( QRegExp(">"), "&gt;" );
  return s;
}

/**
 * Generate HTML with general info on the dictionary, for example
 * description, size, etc.
 */
QString ZBEDic::getDictFrontpage( StaticDictionary *d )
{
  QString infoString;
  
  infoString.append( "<h3>" );
  infoString.append( d->getName() );
  infoString.append( "</h3>" );
  infoString.append( "File: <i>" );
  infoString.append( d->getFileName() );
  infoString.append( "</i><br><hr>" );

  string maintainer;
  d->getProperty( "maintainer", maintainer );
  string copyright;
  d->getProperty( "copyright", copyright );
  string description;
  d->getProperty( "description", description );

  if( copyright.size() != 0 ) {
    infoString.append( escapeString( copyright ) );  
    infoString.append( "<br>" );  
  }
  
  if( description.size() != 0 ) {
    infoString.append( escapeString( description ) );  
    infoString.append( "<br>" );  
  }

  if( maintainer.size() != 0 ) {
    infoString.append( "<i>Report any problems with the dictionary to " );
    infoString.append( escapeString( maintainer ) );  
    infoString.append( "</i><br>" );  
  }  
  
  // Show comment lines
  for( int i = 0; i < 20; i++ ) {
    char cname[10];
    sprintf( cname, "comment%02d", i );
    string commentLine;
    d->getProperty( cname, commentLine );
    if( commentLine.size() == 0 ) break;
    infoString.append( escapeString( commentLine ) + " " );    
  }

  string items;
  d->getProperty( "items", items );  
  string builddate;
  d->getProperty( "builddate", builddate );
  string dicsize;
  d->getProperty( "dict-size", dicsize );
  if( items.size() != 0 || builddate.size() != 0 || dicsize.size() != 0 ) {
    infoString.append( "<hr>Items in dictionary: " );  
    infoString.append( items.c_str() );  
    infoString.append( "<br>Build date: <i>" );  
    infoString.append( builddate.c_str() );  
    infoString.append( "</i><br>Size (uncompressed): " );  
    infoString.append( dicsize.c_str() );  
    infoString.append( " bytes</br>" );  
  }

  return infoString;
}
